package cn.bs.bsmanagebasic.service.impl;

import cn.bs.bsmanagebasic.common.util.CacheTool;
import cn.bs.bsmanagebasic.config.RabbitConfig;
import cn.bs.bsmanagebasic.entity.ExExpressType;
import cn.bs.bsmanagebasic.entity.StoreShelf;
import cn.bs.bsmanagebasic.entity.delivery.Delivery;
import cn.bs.bsmanagebasic.entity.delivery.EsDeliveryQueryCondition;
import cn.bs.bsmanagebasic.entity.dto.EsDeliveryDto;
import cn.bs.bsmanagebasic.entity.filter.DeliveryFilter;
import cn.bs.bsmanagebasic.service.ConfigurationService;
import cn.bs.bsmanagebasic.service.DeliveryESService;
import cn.bs.bsmanagebasic.service.ExpressService;
import cn.bs.bsmanagebasic.service.StoreShelfService;
import cn.bs.bsmanagebasic.utils.StringsUtil;
import cn.bs.bsmanagecommon.util.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.google.gson.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.lang.reflect.Type;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAccessor;
import java.util.*;

/**
 * @author sunjj
 * @version 1.0.0
 * @ClassName DeliverySyncESServiceImpl.java
 * @Description Delivery ES服务相关（数据同步，查询等），
 * 在‘com.parksong.basics.aspect.EnableESAop’进行切面处理，
 * 根据配置（es-server.enable）判断是否启用ES服务，
 * 如果不启用ES服务,则所有方法不会被执行
 * @createTime 2019年10月21日 11:13:00
 */
@Service
@Slf4j
public class DeliveryESServiceImpl implements DeliveryESService {

    /**
     * delivery status 在redis中的有效期配置的Key
     */
    private static final String DELIVERY_STATUS_CACHE_PERIOD = "deliveryStatusCachePeriod";

    @Autowired
    private ConfigurationService configurationService;

    @Autowired
    private RabbitService rabbitService;

    @Autowired
    private RabbitConfig rabbitConfig;

    @Autowired
    private ExpressService expressService;

    @Autowired
    private StoreShelfService storeShelfService;


    @Override
    public void insert(Delivery delivery) {

    }

    @Override
    public void update(EsDeliveryDto dto) {
        try {
            // 补全要更新的信息
            esDeliveryDtoMakeUp(dto);
            // 将某些状态的修改暂存到Redis中，用于解决ES的近及时查询问题
            saveUpdateInfoToRedis(dto);

            // TODO Disable ES
            // Map<String, Object> infoMap = new HashMap<>(5);
            // infoMap.put("data", dto);
            // infoMap.put("operator", "update_fz");
            // rabbitService.send(rabbitConfig.getEsUpdateExchange(), rabbitConfig.getEsUpdateKey(), new Gson().toJson(infoMap));
        } catch (Exception e) {
            log.error("es update error --- msg: {}", e.getMessage());
        }
    }

    /**
     * 将更新的信息保存到redis中用于解决ElasticSearch的近实时查询问题
     *
     * @param dto
     */
    private void saveUpdateInfoToRedis(EsDeliveryDto dto) {
        // 默认有效期 5s
        int defaultPeriod = 5;

        if (null != dto) {

            int statusPeriod;
            try {
                String statusPeriodStr = configurationService.getConfigurationByCode(DELIVERY_STATUS_CACHE_PERIOD).getOValue1();
                statusPeriod = StringUtils.isEmpty(statusPeriodStr) ? defaultPeriod : Integer.valueOf(statusPeriodStr);
            } catch (Exception e) {
                statusPeriod = defaultPeriod;
            }

            if (null != dto.getStatus()) {
                try {
                    CacheTool cacheTool = CacheTool.initDeliveryStatusTool();
                    String key = dto.getStoreId() + ":" + dto.getId();
                    cacheTool.setStringandDatakey(key, dto.getStatus().toString(), statusPeriod);
                } catch (Exception e) {
                }
            }

        }
    }

    /**
     * @description 完善EsDelivertDto信息
     * @param dto
     */
    private void esDeliveryDtoMakeUp(EsDeliveryDto dto) {
        if (null != dto.getExpId()) {
            ExExpressType express = expressService.getExpressById(Long.valueOf(dto.getExpId()));
            dto.setExpName(express.getName());
        }

        if (null != dto.getShelfId()) {
            StoreShelf storeShelf = storeShelfService.getById(dto.getShelfId());
            dto.setShelfName(storeShelf.getShelfName());
        }

        if (!StringUtils.isEmpty(dto.getExpcode()) && StringUtils.isEmpty(dto.getReverseExpCode())) {
            dto.setReverseExpCode(StringsUtil.stringReverse(dto.getExpcode()));
        }
        if (!StringUtils.isEmpty(dto.getUserPhone()) && StringUtils.isEmpty(dto.getReverseUserPhone())) {
            dto.setReverseUserPhone(StringsUtil.stringReverse(dto.getUserPhone()));
        }
    }

    @Override
    public ResponsePage selectByFilter(DeliveryFilter filter) {
        EsDeliveryQueryCondition condition = new EsDeliveryQueryCondition();

        if (null != filter.getStoreIds()) {
            condition.setStoreIds(filter.getStoreIds());
        }
        condition.setTimeType(filter.getTimeType());
        if (null != filter.getStartTime()) {
            condition.setBeginTime(Date.from(filter.getStartTime().atZone(ZoneId.systemDefault()).toInstant()));
        }
        if (null != filter.getEndTime()) {
            condition.setEndTime(Date.from(filter.getEndTime().atZone(ZoneId.systemDefault()).toInstant()));
        }
        if (null != filter.getExpId()) {
            condition.setExpId(filter.getExpId());
        }
        if (null != filter.getExpCode()) {
            condition.setExpcode(filter.getExpCode());
        }
        if (null != filter.getUserPhone()) {
            condition.setUserPhone(filter.getUserPhone());
        }
        if (null != filter.getShelfCode()) {
            condition.setShelfCode(filter.getShelfCode());
        }
        if (null != filter.getCount()) {
            condition.setBegin(filter.getCount().intValue());
        }
        if (null != filter.getStatus()) {
            condition.setStatus(new int[]{filter.getStatus().intValue()});
        }
        if(null != filter.getReturnSignStatus()){
            condition.setReturnSignStatus(filter.getReturnSignStatus().intValue());
        }
        condition.setPageSize(filter.getPageSize());

        Map<String, Object> parameter = new HashMap<>(5);
        parameter.put("data", condition);
        parameter.put("token", "");
        String esServerUrl = configurationService.getConfigurationByCode("esServerUrl").getOValue1();
//        String esServerUrl = "http://192.168.1.152:9001";
        String response = HttpUtil.postData(esServerUrl + "/delivery/search",
                JSON.toJSONString(parameter),
                "application/json", 3000);
        if (!Strings.isBlank(response)) {
            Gson gson = new GsonBuilder().registerTypeAdapter(LocalDateTime.class,
                    (JsonDeserializer<LocalDateTime>) (json, type, jsonDeserializationContext) ->
                            ZonedDateTime.parse(json.getAsJsonPrimitive().getAsString(),
                                    DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSSXXXX")).toLocalDateTime()).create();
            ResponseBean responseBean = gson.fromJson(response, ResponseBean.class);
            if (responseBean.getCode() == 0) {
                updateStatusByRedis(responseBean.getData().getList());
                return responseBean.getData();
            } else {
                log.warn("ES 查询失败，查询结果：{}", response);
            }
        }
        return null;
    }

    @Override
    public Long countByFilter(DeliveryFilter filter) {
        return null;
    }

    private void updateStatusByRedis(List<Delivery> list) {
        if (null != list && list.size() > 0) {
            CacheTool cacheTool = CacheTool.initDeliveryStatusTool();
            for (Delivery delivery : list) {
                try {
                    String key = delivery.getStoreId() + ":" + delivery.getId();
                    String status = cacheTool.getStringKey(key);
                    if (!Strings.isBlank(status)) {
                        delivery.setStatus(Integer.valueOf(status));
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

}
